@@ -1,7 +1,7 @@
 ////////////////////////////////////////////////////////////
 //
 // SFML - Simple and Fast Multimedia Library
-// Copyright (C) 2007-2017 Laurent Gomila (laurent@sfml-dev.org)
+// Copyright (C) 2007-2024 Laurent Gomila (laurent@sfml-dev.org)
 //
 // This software is provided 'as-is', without any express or implied warranty.
 // In no event will the authors be held liable for any damages arising from the use of this software.
@@ -22,8 +22,7 @@
 //
 ////////////////////////////////////////////////////////////
 
-#ifndef SFML_MAIN_HPP
-#define SFML_MAIN_HPP
+#pragma once
 
 ////////////////////////////////////////////////////////////
 // Headers
@@ -33,11 +32,8 @@
 
 #if defined(SFML_SYSTEM_IOS)
 
-    // On iOS, we have no choice but to have our own main,
-    // so we need to rename the user one and call it later
-    #define main sfmlMain
+// On iOS, we have no choice but to have our own main,
+// so we need to rename the user one and call it later
+#define main sfmlMain
 
 #endif
-
-
-#endif // SFML_MAIN_HPP
